@@ -10,7 +10,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -55,7 +55,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
+
+#if !GTK_CHECK_VERSION(2,14,0)
 #include <libgnome/gnome-help.h>
+#endif
 
 #include "rb-file-helpers.h"
 #include "rb-shell-preferences.h"
@@ -87,7 +90,6 @@ void rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 						      RBShellPreferences *shell_preferences);
 static void rb_shell_preferences_toolbar_style_cb (GtkComboBox *box,
 						   RBShellPreferences *preferences);
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 static void rb_shell_preferences_player_backend_cb (GtkToggleButton *button,
 						    RBShellPreferences *preferences);
 static void rb_shell_preferences_album_crossfade_cb (GtkToggleButton *button,
@@ -97,7 +99,6 @@ static void rb_shell_preferences_transition_duration_cb (GtkRange *range,
 static void rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
 							 RBShellPreferences *preferences);
 static void update_playback_prefs_sensitivity (RBShellPreferences *preferences);
-#endif
 
 enum
 {
@@ -122,13 +123,12 @@ struct RBShellPreferencesPrivate
 	GtkWidget *first_seen_check;
 	GtkWidget *quality_check;
 	GtkWidget *year_check;
+	GtkWidget *location_check;
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 	GtkWidget *xfade_backend_check;
 	GtkWidget *album_crossfade_check;
 	GtkWidget *transition_duration;
 	GtkWidget *network_buffer_size;
-#endif
 
 	GSList *browser_views_group;
 
@@ -157,7 +157,14 @@ help_cb (GtkWidget *widget,
 {
 	GError *error = NULL;
 
+#if GTK_CHECK_VERSION(2,14,0)
+	gtk_show_uri (gtk_widget_get_screen (widget),
+		      "ghelp:rhythmbox?prefs",
+		      gtk_get_current_event_time (),
+		      &error);
+#else
 	gnome_help_display ("rhythmbox.xml", "prefs", &error);
+#endif
 
 	if (error != NULL) {
 		rb_error_dialog (NULL,
@@ -238,6 +245,8 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 		glade_xml_get_widget (xml, "year_check");
 	shell_preferences->priv->first_seen_check =
 		glade_xml_get_widget (xml, "first_seen_check");
+	shell_preferences->priv->location_check =
+		glade_xml_get_widget (xml, "location_check");
 
 	/* browser options */
 	rb_glade_boldify_label (xml, "browser_views_label");
@@ -267,7 +276,6 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				    shell_preferences);
 	g_object_unref (xml);
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
 	/* playback preferences */
 	xml = rb_glade_xml_new ("playback-prefs.glade",
 				"playback_prefs_box",
@@ -311,7 +319,6 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 				  glade_xml_get_widget (xml, "playback_prefs_box"),
 				  gtk_label_new (_("Playback")));
 	g_object_unref (xml);
-#endif
 
 	eel_gconf_notification_add (CONF_PLAYER_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
@@ -482,6 +489,8 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_BITRATE";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->first_seen_check))
 		colname = "RHYTHMDB_PROP_FIRST_SEEN";
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->location_check))
+		colname = "RHYTHMDB_PROP_LOCATION";
 	else
 		g_assert_not_reached ();
 
@@ -530,6 +539,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 	char *columns;
 	GSList  *l;
 	gint view, i;
+	gboolean b;
+	float duration;
+	int buffer_size;
 
 	shell_preferences->priv->loading = TRUE;
 
@@ -571,6 +583,9 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->quality_check,
 							 columns, "RHYTHMDB_PROP_BITRATE");
+		rb_shell_preferences_sync_column_button (shell_preferences,
+			       				 shell_preferences->priv->location_check,
+							 columns, "RHYTHMDB_PROP_LOCATION");
 	}
 
 	g_free (columns);
@@ -594,28 +609,20 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 					   G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
 					   shell_preferences);
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
-	{
-		gboolean b;
-		float duration;
-		int buffer_size;
+	/* player preferences */
+	b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
 
-		/* player preferences */
-		b = eel_gconf_get_boolean (CONF_PLAYER_USE_XFADE_BACKEND);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->xfade_backend_check), b);
+	b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
 
-		b = eel_gconf_get_boolean (CONF_PLAYER_TRANSITION_ALBUM_CHECK);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (shell_preferences->priv->album_crossfade_check), !b);
+	duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
+	gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
 
-		duration = eel_gconf_get_float (CONF_PLAYER_TRANSITION_TIME);
-		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->transition_duration), duration);
+	buffer_size = eel_gconf_get_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE);
+	gtk_range_set_value (GTK_RANGE (shell_preferences->priv->network_buffer_size), buffer_size);
 
-		buffer_size = eel_gconf_get_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE);
-		gtk_range_set_value (GTK_RANGE (shell_preferences->priv->network_buffer_size), buffer_size);
-
-		update_playback_prefs_sensitivity (shell_preferences);
-	}
-#endif
+	update_playback_prefs_sensitivity (shell_preferences);
 
 	shell_preferences->priv->loading = FALSE;
 }
@@ -656,8 +663,6 @@ rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
 	eel_gconf_set_integer (CONF_UI_BROWSER_VIEWS, index);
 }
 
-#ifdef HAVE_GSTREAMER_0_10_XFADE
-
 static void
 update_playback_prefs_sensitivity (RBShellPreferences *preferences)
 {
@@ -711,4 +716,3 @@ rb_shell_preferences_network_buffer_size_cb (GtkRange *range,
 	eel_gconf_set_integer (CONF_PLAYER_NETWORK_BUFFER_SIZE, (int)v);
 }
 
-#endif
